<?php
/**
 * CleverReach WordPress Integration.
 *
 * @package CleverReach
 */

namespace CleverReach\WordPress\Controllers;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

use CleverReach\WordPress\Components\InfrastructureServices\Config_Service;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\Entity\AuthInfo;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\Proxy;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\Proxy\AuthProxy;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Interfaces\Required\Configuration;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\Sync\RefreshUserInfoTask;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Exceptions\BadAuthInfoException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WordPress\IntegrationCore\Infrastructure\TaskExecution\Exceptions\QueueStorageUnavailableException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\TaskExecution\Queue;
use CleverReach\WordPress\IntegrationCore\Infrastructure\TaskExecution\QueueItem;
use CleverReach\WordPress\Components\Utility\Helper;

/**
 * Class Clever_Reach_Callback_Controller
 *
 * @package CleverReach\WordPress\Controllers
 */
class Clever_Reach_Callback_Controller extends Clever_Reach_Base_Controller {

	/**
	 * Run the request
	 */
	public function run() {
		$this->validate_request();

		if ( $this->get_param( 'refresh' ) ) {
			$auth_info = $this->get_auth_info( true );
			$this->refresh_tokens( $auth_info );
		} else {
			$auth_info = $this->get_auth_info( false );
			$this->queue_refresh_user_task( $auth_info );
		}

		include dirname( __DIR__ ) . '/resources/views/close-frame.php';
	}

	/**
	 * Validates the request
	 */
	private function validate_request() {
		$code = $this->get_param( 'code' );

		if ( empty( $code ) ) {
			$this->die_json(
				array(
					'status'  => false,
					'message' => __( 'Wrong parameters. Code not set.' ),
				)
			);
		}
	}

	/**
	 * Returns auth info
	 *
	 * @param string $refresh_tokens Do refresh tokens.
	 *
	 * @return AuthInfo
	 */
	private function get_auth_info( $refresh_tokens ) {
		$result = null;
		$code   = $this->get_param( 'code' );

		$proxy        = ServiceRegister::getService( AuthProxy::CLASS_NAME );
		$redirect_url = $refresh_tokens ? Helper::get_refresh_auth_redirect_url() : Helper::get_auth_redirect_url();

		try {
			$result = $proxy->getAuthInfo( $code, $redirect_url );
		} catch ( BadAuthInfoException $e ) {
			$this->die_json(
				array(
					'status'  => false,
					'message' => $e->getMessage(),
				)
			);
		}

		return $result;
	}

	/**
	 * Refreshes user access tokens
	 *
	 * @param AuthInfo $auth_info User auth info object.
	 */
	private function refresh_tokens( $auth_info ) {
		/** @var Config_Service $config */
		$config = ServiceRegister::getService( Configuration::CLASS_NAME );
		/** @var Proxy $proxy */
		$proxy  = ServiceRegister::getService( Proxy::CLASS_NAME );

		try {
			$user_info = $proxy->getUserInfo( $auth_info->getAccessToken() );
		} catch ( \Exception $e ) {
			$user_info = array();
		}

		$local_info = $config->getUserInfo();

		if ( isset( $user_info['id'] ) && $user_info['id'] === $local_info['id'] ) {
			$config->setAuthInfo( $auth_info );
		}
	}

	/**
	 * Queues refresh user info task
	 *
	 * @param AuthInfo $auth_info Auth info object.
	 */
	private function queue_refresh_user_task( $auth_info ) {
		/** @var Queue $queue */
		$queue  = ServiceRegister::getService( Queue::CLASS_NAME );
		/** @var Config_Service $config */
		$config = ServiceRegister::getService( Configuration::CLASS_NAME );

		try {
			$config->setAuthInfo( $auth_info );
			$queue->enqueue(
				$config->getQueueName(),
				new RefreshUserInfoTask( $auth_info ),
				'',
				QueueItem::PRIORITY_HIGH
			);
		} catch ( QueueStorageUnavailableException $e ) {
			$this->die_json(
				array(
					'status'  => false,
					'message' => $e->getMessage(),
				)
			);
		}
	}
}
