<?php

namespace CleverReach\WordPress\IntegrationCore\BusinessLogic\Sync;

use CleverReach\WordPress\IntegrationCore\BusinessLogic\Proxy\FormProxy;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\RepositoryRegistry;
use CleverReach\WordPress\IntegrationCore\BusinessLogic\Scheduler\Models\ConfigIntervalSchedule;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Exceptions\InvalidConfigurationException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Logger\Logger;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ORM\Exceptions\QueryFilterInvalidParamException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ORM\Exceptions\RepositoryClassException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ORM\Exceptions\RepositoryNotRegisteredException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ORM\QueryFilter\Operators;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ORM\QueryFilter\QueryFilter;
use CleverReach\WordPress\IntegrationCore\Infrastructure\ServiceRegister;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpAuthenticationException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpCommunicationException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\HttpRequestException;
use CleverReach\WordPress\IntegrationCore\Infrastructure\Utility\Exceptions\RefreshTokenExpiredException;

/**
 * Class FormSyncTask
 *
 * @package CleverReach\WordPress\IntegrationCore\BusinessLogic\Sync
 */
class FormSyncTask extends BaseSyncTask
{
    /**
     * @var FormProxy
     */
    private $formProxy;

    /**
     * @inheritDoc
     *
     * @throws InvalidConfigurationException
     * @throws HttpAuthenticationException
     * @throws HttpCommunicationException
     * @throws HttpRequestException
     * @throws RefreshTokenExpiredException
     * @throws RepositoryClassException
     * @throws RepositoryNotRegisteredException
     * @throws QueryFilterInvalidParamException
     *
     */
    public function execute()
    {
        $this->reportProgress(20);

        try {
            $integrationId = $this->getConfigService()->getIntegrationId();
            $formName = $this->getConfigService()->getIntegrationFormName();
            if (!$this->formExists($integrationId, $formName)) {
                $this->reportProgress(50);
                $this->getFormProxy()->createForm($integrationId, $formName);
            }

            $this->reportProgress(70);
            $this->scheduleFormCacheTaskIfNotExist();
        } catch (\Exception $e) {
            Logger::logError('Failed to create form. Error: ' . $e->getMessage());
        }

        $this->reportProgress(100);
    }

    /**
     * Checks if form exists for given integration
     *
     * @param int $integrationId CleverReach group name
     * @param string $formName CleverReach form name
     *
     * @return bool
     *
     * @throws InvalidConfigurationException
     * @throws HttpAuthenticationException
     * @throws HttpCommunicationException
     * @throws HttpRequestException
     * @throws RefreshTokenExpiredException
     */
    private function formExists($integrationId, $formName)
    {
        $existingForms = $this->getFormProxy()->getFormList();
        $this->reportAlive();
        foreach ($existingForms as $form) {
            if ($form['name'] === $formName && (int)$form['customer_tables_id'] === $integrationId) {
                return true;
            }
        }

        return false;
    }

    /**
     * Creates minute schedule if not exists for FormCacheTask
     *
     * @throws RepositoryClassException
     * @throws RepositoryNotRegisteredException
     * @throws QueryFilterInvalidParamException
     */
    private function scheduleFormCacheTaskIfNotExist()
    {
        $scheduleRepository = RepositoryRegistry::getScheduleRepository();
        $filter = new QueryFilter();
        $filter->where('context', Operators::EQUALS, $this->getConfigService()->getContext())
            ->where('taskType', Operators::EQUALS, FormCacheSyncTask::getClassName());

        $scheduledFilterCacheTasks = $scheduleRepository->select($filter);
        if (empty($scheduledFilterCacheTasks)) {
            $minuteSchedule = new ConfigIntervalSchedule(
                new FormCacheSyncTask(),
                $this->getConfigService()->getQueueName(),
                $this->getConfigService()->getContext(),
                FormCacheSyncTask::SCHEDULE_EXECUTION_INTERVAL_CONFIG_KEY
            );

            $minuteSchedule->setNextSchedule();
            $scheduleRepository->save($minuteSchedule);
        }
    }

    /**
     * @return FormProxy
     */
    private function getFormProxy()
    {
        if ($this->formProxy === null) {
            $this->formProxy = ServiceRegister::getService(FormProxy::CLASS_NAME);
        }

        return $this->formProxy;
    }
}
